
from expon.core.exp import EXP
from expon.core.params import Params
from expon.core.metric import Metric

def test():

    exp = EXP()
    params = Params()
    params.lamb = 1
    params.learning_rate = 0.001
    params.batch_size = 512
    print(params)
    exp.set_params(params)

    loss = Metric('loss', 'epoch')
    acc = Metric('loss')
    exp.add_metric(loss)
    exp.add_metric(acc)

    exp.set_seed(123)

    for i in range(0, 100):
        loss.update(1-0.01*i)
        acc.update(0.01*i)

    exp.save(output_format='html')

if __name__ == '__main__':
    test()
    


import os

from .file import get_root_path

import pkg_resources

DEFAULT_CSS = pkg_resources.resource_filename('expon', 'asset/default.css')

GITHUB_CSS = pkg_resources.resource_filename('expon', 'asset/github.css')

CURRENT_DIR = get_root_path()

# default work directory ./EXP
ROOT_DIR = os.path.join(CURRENT_DIR, 'EXP')

# default work directory ./EXP/run
WORKSPACE_DIR = os.path.join(ROOT_DIR, 'run')

# experiment directory ./EXP/run/experiment
EXPERIMENT_DIR = None




import json
import numpy as np

from  ..utils import vis
from ..utils import gconfig
from ..utils.file import *

class Metric():
    def __init__(self, value_label, step_label=None, decimal=6, draw=False):
        if step_label is None:
            step_label = 'step'

        self.value= 0
        self.avg = 0
        self.sum = 0
        self.step = 0
        self.history = []

        self.value_label = value_label
        self.step_label = step_label
        self.decimal = decimal

        self.draw = draw

    def update(self, value, n=1):
        self.value = value
        self.step += n
        self.sum += value * n
        self.avg = self.sum / self.step
        self.history.append([self.step, self.value])

    def visualization(self, show=True, filename=None, save_dir=None):
        if filename is None:
            filename = self.value_label + '-' + self.step_label

        if '.png' not in filename:
            filename += '.png'

        if save_dir is None:
            create_dir(gconfig.EXPERIMENT_DIR)
            file_path = os.path.join(gconfig.EXPERIMENT_DIR, filename)
        else:
            create_dir(save_dir)
            file_path = os.path.join(save_dir, filename)
        
        history = np.array(self.history)
        self.file_path = vis.draw_line(history[:, 0], 
                                        history[:, 1], 
                                        file_path, 
                                        self.step_label, self.value_label, show)

    def save2json(self, filename=None):
        if filename is None:
            filepath = os.path.join(gconfig.EXPERIMENT_DIR, self.value_label+'.json')
        else:
            if '.json' not in filename:
                filename = filename + '.json'
            filepath = os.path.join(gconfig.EXPERIMENT_DIR, filename)

        create_dir(gconfig.EXPERIMENT_DIR)
        with open(filepath, 'a+') as f:
            output = {self.value_label+'-'+self.step_label: self.history}
            f.write(dict2jsonstr(output))





import pprint as pp
from typing import Any
from collections import OrderedDict

from ..utils.file import *
from ..utils import gconfig

class Params:
    def __init__(self):
        self._param_dict = OrderedDict()

    def load_json(self, filepath):
        if '.json' not in filepath:
            filepath += '.json'
        if not check_file(filepath):
            raise Exception(filepath + ' dose not exist!')
        with open(filepath, 'r') as f:
            file_dict = jsonstr2dict(f.read())
            for k, v in file_dict.items():
                if k in ['git_id', 'start_time']:
                    pass
                else:
                    setattr(self, k, v)

    def save2json(self, filename=None):
        if filename is None:
            filepath = os.path.join(gconfig.EXPERIMENT_DIR, 'params.json')
        else:
            if '.json' not in filename:
                filename = filename + '.json'
            filepath = os.path.join(gconfig.EXPERIMENT_DIR, filename)

        create_dir(gconfig.EXPERIMENT_DIR)
        with open(filepath, 'w') as f:
            f.write(dict2jsonstr(self._param_dict))


    def __setattr__(self, name: str, value: Any) -> None:
        if name.startswith("_"):
            super().__setattr__(name, value)
        else:
            self._param_dict[name] = value

    def __setitem__(self, key, value):
        key = str(key)
        self.__setattr__(key, value)

    def __getattr__(self, item):
        if item not in self._param_dict:
            raise AttributeError(item)
        return self._param_dict[item]

    def __getitem__(self, item):
        return self._param_dict[item]

    def __repr__(self):
        return "{}".format(self.__class__.__name__) + pp.pformat([(k, v) for k, v in self._param_dict.items()])


import time
import datetime

def get_current_time():
    current_time = datetime.datetime.fromtimestamp(int(time.time())).strftime('%Y-%m-%d_%H:%M:%S')
    return current_time


if __name__ == "__main__":
    print(get_current_time())


import sys
class Logger(object):
    def __init__(self, filename='default.log', stream=sys.stdout, mode='w'):
        self.terminal = stream
        self.log = open(filename, mode)
 
    def write(self, message):
        self.terminal.write(message)
        self.log.write(message)
 
    def flush(self):
        pass
   

def terminal_to_log(log_filepath):
    sys.stdout = Logger(log_filepath, sys.stdout)
    sys.stderr = Logger(log_filepath, sys.stderr, mode='a')
    # sys.stderr = Logger('a.log_file', sys.stderr)


import os

import random
from collections import OrderedDict

import numpy as np
import torch

from ..utils.file import *
from ..utils.time import get_current_time
from ..utils.git import *
from ..utils import gconfig
from ..utils import md
from ..log.logging import terminal_to_log

class EXP():
    def __init__(self, workspace=None, exp_name=None, exp_description=None, log=True, check_git=True):
        if check_git:
           check_working_tree()

        if workspace is None:
            workspace = 'run'
        if exp_description is None:
            exp_description = 'empty'

        self.workspace = workspace
        self.exp_name = exp_name
        self.exp_description = exp_description
        self.metrics = []
        self.info = {}
        self.exp_seed = None

        self._start()
        self._set_git_id()
        self._set_dir()

        if log:
            self.log()

    def add_info(self, info_dict):
        self.info.update(info_dict)

    def log(self):
        file_path = os.path.join(self.experiment_dir, 'out.log')
        create_dir_from_file(file_path)
        terminal_to_log(file_path)

    def get_info(self, key):
        return self.info[key]
    
    def set_seed(self, seed=None):
        '''
        Set experiment seed (random, numpy and torch). 
        If seed is None then randomly choose 0~1000 as seed.
        '''

        if seed is None:
            seed = random.randint(0,1000)
        else:
            assert type(seed) == int

        random.seed(seed)
        np.random.seed(seed)
        torch.manual_seed(seed)
        torch.random.manual_seed(seed)
        if torch.cuda.is_available():
            torch.cuda.manual_seed(seed)
            torch.backends.cudnn.deterministic = True
        self.exp_seed = seed

    def save(self, output_format='md', show_metric=False):
        '''
        Save the params, metrics.
        '''
        assert output_format in ['md', 'markdown', 'html']

        file_path = os.path.join(self.experiment_dir, 'results')
        create_dir_from_file(file_path)

        self.end_time = get_current_time()
        # save params
        self.params.save2json()

        # save metrics
        for metric in self.metrics:
            if metric.history:
                if metric.draw:
                    metric.visualization(show=show_metric)
                metric.save2json()

        md_text = self.results2md()

        if output_format in ['md', 'markdown']:
            md.save2md(md_text, file_path)
        else:
            md.save2html(md_text, file_path)
        
        print('exp infomation saved to', self.experiment_dir)

    def results2md(self):
        results = []
        md.add_title(results, self.exp_name, level=3)

        md.add_title(results, 'Description', level=5)
        md.add_text(results, self.exp_description)

        md.add_title(results, 'Time', level=5)
        time_text = self.start_time+' --- '+self.end_time
        md.add_text(results, time_text)
        #TODO Time used

        if self.exp_seed is not None:
            md.add_title(results, 'Seed', level=5)
            md.add_text(results, str(self.exp_seed))
        
        if self.metrics: 
            md.add_title(results, 'Metric', level=5)
            for metric in self.metrics:
                metric_text = metric.value_label+':   final: '+str(round(metric.value, metric.decimal))+', avg: '+str(round(metric.avg, metric.decimal))+', sum: '+str(round(metric.sum, metric.decimal))
                md.add_text(results, metric_text)

        for metric in self.metrics:
            if metric.history:
                if metric.draw:
                    md.add_image(results, metric.value_label, metric.file_path)

        if self.info:
            md.add_title(results, 'Info', level=5)
            for (key, value) in self.info.items():
                md.add_title(results, key, level=6)
                md.add_text(results, str(value))

        md.add_title(results, 'Params', level=5)
        md.add_code(results, dict2jsonstr(self.params._param_dict))

        return results

    def set_params(self, params):
        self.params = params

    def add_metric(self, metric):
        self.metrics.append(metric)

    def _start(self):
        self.start_time = get_current_time()
        if self.exp_name is None:
            self.exp_name = self.start_time

    def _set_dir(self):
        if self.workspace is None:
            gconfig.WORKSPACE_DIR = os.path.join(gconfig.ROOT_DIR, 'run')
        else:
            gconfig.WORKSPACE_DIR = os.path.join(gconfig.ROOT_DIR, self.workspace)

        if self.exp_name is None:
            gconfig.EXPERIMENT_DIR = os.path.join(gconfig.WORKSPACE_DIR, self.start_time)
        else:
            gconfig.EXPERIMENT_DIR = os.path.join(gconfig.WORKSPACE_DIR, self.exp_name)

        if check_dir(gconfig.EXPERIMENT_DIR):
            raise Exception(gconfig.EXPERIMENT_DIR + 'already exits!')

        self.work_dir = gconfig.WORKSPACE_DIR
        self.experiment_dir = gconfig.EXPERIMENT_DIR

    def _set_git_id(self):
        self.git_id = get_git_revision_short_hash()




import os
import matplotlib.pyplot as plt


from .file import *

def draw_line(x, y, filepath, xlabel=None, ylabel=None, show=False):

    fig, ax = plt.subplots(figsize=(4,4))
    ax.plot(x, y)
    if xlabel is not None and ylabel is not None:
        ax.set_title(ylabel + ' vs. ' + xlabel)
        ax.set_xlabel(xlabel)
        ax.set_ylabel(ylabel)

    ax.grid(which='minor', axis='both')
    fig.savefig(filepath)

    if show:
        plt.show()
    return filepath



import subprocess

def check_working_tree():
    try:
        subprocess.check_output(['git', 'diff', '--exit-code'])

    except subprocess.CalledProcessError as e:
        print('called process error')
        out_bytes = e.output       # Output generated before error
        code = e.returncode   # Return code
        raise Exception('Your working tree is not empty, please commit all changes.')
    else:
        return True

def get_git_revision_short_hash():
    try:
        commit_id = subprocess.check_output(['git', 'rev-parse', '--short', 'HEAD']).decode('ascii').strip()
    except subprocess.CalledProcessError:
        raise Exception('Somthing is wrong with your git workspace!')
    else:
        return commit_id



import codecs
import pkgutil

import markdown

from .file import *
from . import gconfig


SPLIT = '\n\n'

def add_text(source, text):
    return source.append(text+SPLIT)

def add_title(source, title, level=1):
    assert type(level) == int
    assert level in [1, 2, 3, 4, 5, 6]
    title = '#'*level+' '+title
    source.append(title+SPLIT)
    return source

def add_code(source, code, lang='json'):
    code = '```'+lang+'\n'+code+'\n'+'```'
    source.append(code+SPLIT)
    return source

def add_quote(source, quote):
    quote ='> '+quote
    source.append(quote+SPLIT)
    return source

def add_line(source):
    source.append('------'+SPLIT)
    return source

def add_image(source, title, image_path):
    if check_file(image_path):
        image_str = '!['+title+']('+image_path+')'
        source.append(image_str+SPLIT)
    else:
        raise Exception('Image dose not exit!')
    return source

def save2md(source, file_path):
    if '.md' not in file_path:
        file_path += '.md'

    if check_file(file_path):
        raise Exception(file_path+'already exits!')

    md_str = ''.join(source)
    # with open(file_path, 'w') as f:
    #     f.write(source)

    with codecs.open(file_path, "w", encoding="utf-8") as f:
        f.write(md_str)

def md2html(mdstr):
    exts = ['markdown.extensions.extra', 'markdown.extensions.codehilite','markdown.extensions.tables','markdown.extensions.toc']

    html = '''
    <html lang="zh-cn">
    <head>
    <meta content="text/html; charset=utf-8" http-equiv="content-type" />
    <link href="%s" rel="stylesheet">
    <link href="%s" rel="stylesheet">
    </head>
    <body>
    %s
    </body>
    </html>
    '''

    ret = markdown.markdown(mdstr,extensions=exts)
    return html % (gconfig.DEFAULT_CSS, gconfig.GITHUB_CSS, ret)

def check_css():
    if not check_file(gconfig.DEFAULT_CSS):
        raise Exception(gconfig.DEFAULT_CSS + ' does not exit!')
        # data = pkgutil.get_data('expon', 'asset/default.css').decode('utf-8')
        # with open(gconfig.DEFAULT_CSS, 'w') as f:
        #     f.write(data)
        
    if not check_file(gconfig.GITHUB_CSS):
        raise Exception(gconfig.GITHUB_CSS + ' does not exit!')
        # data = pkgutil.get_data('expon', 'asset/github.css').decode('utf-8')
        # with open(gconfig.GITHUB_CSS, 'w') as f:
        #     f.write(data)

def save2html(source, file_path):
    check_css()
    if '.html' not in file_path:
        file_path += '.html'
    if check_file(file_path):
        raise Exception(file_path+'already exits!')

    md_str = ''.join(source)
    html = md2html(md_str)
    with codecs.open(file_path, "w", encoding="utf-8", errors="xmlcharrefreplace") as f:
        f.write(html)





import os
import json

def jsonstr2dict(json_str):
    return json.loads(json_str)

def dict2jsonstr(dic, is_format=True):
    if is_format:
        return json.dumps(dic, sort_keys=False, indent=4, separators=(',', ': '))
    return json.dumps(dic)

def raise_exception(str):
    raise Exception(str)

def get_root_path():
    return os.getcwd()

def check_file(file_path):
    return os.path.isfile(file_path)
    
def check_dir(dir_path):
    return os.path.isdir(dir_path)

def create_dir(dir_path):
    if not check_dir(dir_path):
        os.makedirs(dir_path)

def create_dir_from_file(file_path):
    file_dir = os.path.split(file_path)[0]
    create_dir(file_dir)


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="expon", # Replace with your own username
    version="0.0.7",
    author="Zhenyu Huang",
    author_email="zyhuang.gm@gmail.com",
    description="Experiment tool for deep learning (PyTorch).",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/hi-zhenyu/expon",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
          'markdown',
          'numpy',
          'matplotlib',
    ],
    python_requires='>=3.6',
    include_package_data=True,
)

# python3 setup.py sdist bdist_wheel

# python3 -m twine upload dist/*


# python3 -m twine upload --repository testpypi dist/*